#include "network.h"
#include <qsocketnotifier.h>

#ifdef WITH_GGZ
Network *Network::self;
#endif

Network::Network()
: QObject()
{
#ifdef WITH_GGZ
	self = this;
	mod = NULL;
#endif
}

Network::~Network()
{
}

void Network::connect()
{
#ifdef WITH_GGZ
	int fd;
	QSocketNotifier *sn;

	mod = ggzmod_new(GGZMOD_GAME);
	ggzmod_set_handler(mod, GGZMOD_EVENT_SERVER, &server_input);
	ggzmod_connect(mod);
	fd = ggzmod_get_fd(mod);

	sn = new QSocketNotifier(fd, QSocketNotifier::Read, this);
	QObject::connect(sn, SIGNAL(activated(int)), SLOT(slotDispatch()));
#endif
}

void Network::disconnect()
{
#ifdef WITH_GGZ
	ggzmod_disconnect(mod);
	ggzmod_free(mod);
#endif
}

void Network::slotInput(int fd)
{
	emit signalInput(fd);
}

#ifdef WITH_GGZ
void Network::server_input(GGZMod *mod, GGZModEvent e, void *data)
{
	ggzmod_set_state(mod, GGZMOD_STATE_PLAYING);
	self->serverInput(*(int*)data);
}
#endif

void Network::serverInput(int fd)
{
	QSocketNotifier *sn;

	sn = new QSocketNotifier(fd, QSocketNotifier::Read, this);
	QObject::connect(sn, SIGNAL(activated(int)), SLOT(slotInput(int)));
}

void Network::slotDispatch()
{
	ggzmod_dispatch(mod);
}

