#ifndef PLAYCANVAS_H
#define PLAYCANVAS_H

#include <qcanvas.h>
#include <qptrlist.h>

#include "level.h"

class QPixmap;
class QTimer;
class Bomb;
class Player;
class Blast;
class Item;
class Exit;
class AI_Player;
class Resource;

class PlayCanvas : public QCanvas
{
	Q_OBJECT
public:
	PlayCanvas(QObject *parent = NULL, const char *name = NULL);
	~PlayCanvas();

	enum Events
	{
		moveEvent,
		fireEvent
	};
	enum Options
	{
		optNone,
		optLeft,
		optRight,
		optDown,
		optUp
	};

	enum TileTypes
	{
		invalid,
		ground,
		wall,
		bridge,
		space,
		itemScree,
		itemBomb,
		itemLive,
		itemPower
	};

	void buildLevel(QString levelfile);
	void drawBackground(void);
	void placeDynamic(void);
	void createPlayer();
	void createMonster();
	bool placeBlast(int x, int y);
	void checkItem(int x, int y);
	void action(int event, int option);
	bool walkableTile(int x, int y);
	void setTheme(QString theme);
	void setPlayers(int players);

	Level *level();

public slots:
	void monsterTimeout();

signals:
	void playerKilled();
	void levelFinished();

protected:
	void timerEvent(QTimerEvent *e);

private:
	int random(int max);
	Item *findItem(int x, int y);
	Level *m_level;

	QPtrList<Bomb> bombList;
	QPtrList<Item> itemList;
	QPtrList<Blast> blastList;
	QPtrList<Player> playerList;
	QPtrList<AI_Player> monsterList;

	QTimer *monsterTimer;

	Player *currPlayer;
	Resource *resource;
	Exit *exit;
	bool m_away;
	int m_players;
};

#endif
