#include "select.h"
#include "playground.h"
#include "playcanvas.h"
#include "resource.h"

#include <qlayout.h>
#include <qframe.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qcombobox.h>

#include <klocale.h>
#include <ksimpleconfig.h>

#include "config.h"

Select::Select(QWidget *parent, const char *name)
: QDialog(parent, name)
{
	QVBoxLayout *vbox;
	QHBoxLayout *hbox, *hbox2;
	QPushButton *ok, *cancel;
	QLabel *desclevel, *desctheme, *descplayers;

	m_playground = NULL;
	m_playcanvas = NULL;

	desclevel = new QLabel(i18n("Level to play"), this);
	desctheme = new QLabel(i18n("Theme to use"), this);
	descplayers = new QLabel(i18n("Number of AI players"), this);

	combolevel = new QComboBox(this);

	KSimpleConfig conf(DATADIR "/levels/levels.list");
	conf.setGroup("levels");
	QStringList l = conf.readListEntry("levels");
	for(QStringList::iterator it = l.begin(); it != l.end(); it++)
	{
		conf.setGroup((*it));
		QString name = conf.readEntry("Name", i18n("(unknown)"));
		combolevel->insertItem(name);
		m_map[name] = (*it);
	}

	combotheme = new QComboBox(this);

	KSimpleConfig conf2(DATADIR "/pics/themes.registry");
	conf2.setGroup("themes");
	QStringList l2 = conf2.readListEntry("themes");
	for(QStringList::iterator it = l2.begin(); it != l2.end(); it++)
	{
		conf2.setGroup((*it));
		QString name = conf2.readEntry("Name", i18n("(unknown)"));
		combotheme->insertItem(name);
		m_map2[name] = (*it);
	}

	comboplayers = new QComboBox(this);
	for(int i = 1; i < 8; i++)
		comboplayers->insertItem(QString("%1").arg(i));

	preview = new QFrame(this);
	preview->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	preview->setFixedSize(128, 128);
	
	xlayout = new QVBoxLayout(preview, 2);

	ok = new QPushButton(i18n("Select"), this);
	cancel = new QPushButton(i18n("Cancel"), this);

	vbox = new QVBoxLayout(this, 5);
	vbox->add(desclevel);
	vbox->add(combolevel);
	vbox->add(desctheme);
	vbox->add(combotheme);
	vbox->add(descplayers);
	vbox->add(comboplayers);
	hbox2 = new QHBoxLayout(vbox, 5);
	hbox2->addStretch(1);
	hbox2->add(preview);
	hbox2->addStretch(1);
	hbox = new QHBoxLayout(vbox, 5);
	hbox->add(ok);
	hbox->add(cancel);

	connect(comboplayers, SIGNAL(activated(int)), SLOT(minimap()));
	connect(combolevel, SIGNAL(activated(int)), SLOT(minimap()));
	connect(combotheme, SIGNAL(activated(int)), SLOT(minimap()));
	connect(ok, SIGNAL(clicked()), SLOT(slotAccept()));
	connect(cancel, SIGNAL(clicked()), SLOT(reject()));

	setCaption(i18n("Select a level"));

	minimap();
}

Select::~Select()
{
}

void Select::slotAccept()
{
	if(m_playground) delete m_playground;
	if(m_playcanvas) delete m_playcanvas;

	accept();
}

void Select::minimap()
{
	if(m_playground) delete m_playground;
	if(m_playcanvas) delete m_playcanvas;

	int oldsize = Resource::tilesize();
	Resource::setTilesize(6);
	m_playcanvas = new PlayCanvas(preview);
	m_playcanvas->setTheme(theme());
	m_playcanvas->setPlayers(players());
	m_playcanvas->buildLevel(level());
	Resource::setTilesize(oldsize);

	m_playground = new Playground(m_playcanvas, preview);
	//m_playground->move(preview->x(), preview->y());
	m_playground->resize(preview->width() - 4, preview->height() - 4);
	xlayout->add(m_playground);
}

QString Select::level()
{
	return m_map[combolevel->currentText()];
}

QString Select::theme()
{
	return m_map2[combotheme->currentText()];
}

int Select::players()
{
	return comboplayers->currentText().toInt();
}

