#include "toplevel.h"
#include "playground.h"
#include "playcanvas.h"
#include "network.h"
#include "kmenubar.h"
#include "kpopupmenu.h"
#include "splash.h"
#include "select.h"
#include "level.h"
#include "resource.h"

#include <kapplication.h>
#include <kdebug.h>
#include <klocale.h>

TopLevel::TopLevel(int ggzmode)
	: KMainWindow()
{
	KPopupMenu *gamemenu;

	m_ggzmode = ggzmode;

	Resource::setTheme("desert");
	Resource::setTilesize(40);

	network = NULL;
	playground = NULL;
	canvas = NULL;
	splash = new Splash(this);
	setCentralWidget(splash);

	gamemenu = new KPopupMenu(this);
	if(!m_ggzmode)
	{
		gamemenu->insertItem(i18n("New"), game_new);
		gamemenu->insertSeparator();
	}
	else
	{
		network = new Network();
		network->connect();
		connect(network, SIGNAL(signalInput(int)), SLOT(slotInput(int)));
	}
	gamemenu->insertItem(i18n("Exit"), game_exit);

	menuBar()->insertItem(i18n("Game"), gamemenu);
	menuBar()->insertItem(i18n("Help"), helpMenu());

	connect(gamemenu, SIGNAL(activated(int)), SLOT(slotMenu(int)));

	setCaption(i18n("Kamikaze"));

	move(kapp->desktop()->width() / 2 - splash->autowidth() / 2,
		kapp->desktop()->height() / 2 - splash->autoheight() / 2);
}

TopLevel::~TopLevel()
{
	if(m_ggzmode)
	{
		network->disconnect();
		delete network;
	}
}

void TopLevel::key(int keycode)
{
	//kdDebug() << "key " << keycode << endl;
	if(!canvas) return;
	switch(keycode)
	{
		case Playground::KeyLeft:
			canvas->action(PlayCanvas::moveEvent, PlayCanvas::optLeft);
			break;
		case Playground::KeyRight:
			canvas->action(PlayCanvas::moveEvent, PlayCanvas::optRight);
			break;
		case Playground::KeyUp:
			canvas->action(PlayCanvas::moveEvent, PlayCanvas::optUp);
			break;
		case Playground::KeyDown:
			canvas->action(PlayCanvas::moveEvent, PlayCanvas::optDown);
			break;
		case Playground::KeyFire:
			canvas->action(PlayCanvas::fireEvent, PlayCanvas::optNone);
			break;
	}
}

void TopLevel::newGame(QString level, QString theme, int players)
{
	if (!canvas && !playground) {
		canvas = new PlayCanvas(this);

		playground = new Playground(canvas, this);
		connect(playground, SIGNAL(key(int)), SLOT(key(int)));
		connect(canvas, SIGNAL(playerKilled()), SLOT(slotEndGame()));
		connect(canvas, SIGNAL(levelFinished()), SLOT(slotEndGame()));

		canvas->setTheme(theme);
		canvas->setPlayers(players);
		canvas->buildLevel(level);
		playground->setFocus();

		splash->hide();
		playground->show();
		setCentralWidget(playground);

		// HACK!!!
		setFixedSize(canvas->level()->width() * Resource::tilesize(),
			canvas->level()->height() * Resource::tilesize() + menuBar()->height());
	}
}

void TopLevel::selectNewGame()
{
	int ret;
	Select sel;
	ret = sel.exec();
	if(ret == QDialog::Accepted)
	{
		newGame(sel.level(), sel.theme(), sel.players());
	}
}

void TopLevel::slotMenu(int id)
{
	switch(id)
	{
		case game_new:
			selectNewGame();
			break;
		case game_exit:
			slotEndGame();
			close();
			break;
	}
}

void TopLevel::setTheme(QString theme)
{
	if(canvas) canvas->setTheme(theme);
}

void TopLevel::slotEndGame()
{
	setCentralWidget(splash);
	splash->show();
	setFixedSize(splash->autowidth(), splash->autoheight());


	if (playground)
		delete playground;
	if (canvas)
		delete canvas;

	playground = NULL;
	canvas = NULL;
}

void TopLevel::keyPressEvent(QKeyEvent *e)
{
	//kdDebug() << "toplevel: key pressed" << endl;
	switch(e->key())
	{
		case Qt::Key_Down:
			key(Playground::KeyDown);
			break;
		case Qt::Key_Up:
			key(Playground::KeyUp);
			break;
		case Qt::Key_Left:
			key(Playground::KeyLeft);
			break;
		case Qt::Key_Right:
			key(Playground::KeyRight);
			break;
		case Qt::Key_Space:
			key(Playground::KeyFire);
			break;
	}
}

void TopLevel::slotInput(int fd)
{
}

