#include "level.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

Level::Level()
{
	m_width = 0;
	m_height = 0;
	m_field = NULL;
	m_occupation = NULL;
}

Level::~Level()
{
	if(m_field)
	{
		for(int i = 0; i < m_width; i++)
			free(m_field[i]);
		free(m_field);
	}
	if(m_occupation)
	{
		for(int i = 0; i < m_width; i++)
			free(m_occupation[i]);
		free(m_occupation);
	}
}

void Level::loadLevel(const char *levelfile)
{
	FILE *f;
	char buf[256];
	char *token;
	int i, j;

	f = fopen(levelfile, "r");
	if(!f) return;

	fgets(buf, sizeof(buf), f);
	buf[strlen(buf) - 1] = 0;
	m_width = atoi(buf);
	fgets(buf, sizeof(buf), f);
	buf[strlen(buf) - 1] = 0;
	m_height = atoi(buf);

	m_field = (int**)malloc(sizeof(int*) * m_width);
	for(i = 0; i < m_width; i++)
	{
		m_field[i] = (int*)malloc(sizeof(int) * m_height);
		for(j = 0; j < m_height; j++)
			m_field[i][j] = invalid;
	}
	m_occupation = (int**)malloc(sizeof(int*) * m_width);
	for(i = 0; i < m_width; i++)
	{
		m_occupation[i] = (int*)malloc(sizeof(int) * m_height);
		for(j = 0; j < m_height; j++)
			m_occupation[i][j] = 0;
	}

	j = 0;
	while(fgets(buf, sizeof(buf), f))
	{
		buf[strlen(buf) - 1] = 0;
		token = strtok(buf, " ");
		i = 0;
		while(token)
		{
			if((i >= 0) && (j >= 0) && (i < m_width) && (j < m_height))
				m_field[i][j] = atoi(token);
			token = strtok(NULL, " ");
			i++;
		}
		j++;
	}

	fclose(f);
}

void Level::setTile(int x, int y, int tiletype)
{
	if(!m_field) return;
	if((x < 0) || (y < 0) || (x >= m_width) || (y >= m_height)) return;
	m_field[x][y] = tiletype;
}

int Level::tile(int x, int y)
{
	if(!m_field) return invalid;
	if((x < 0) || (y < 0) || (x >= m_width) || (y >= m_height)) return invalid;
	return m_field[x][y];
}

int Level::width()
{
	return m_width;
}

int Level::height()
{
	return m_height;
}

int Level::occupied(int x, int y)
{
	if(!m_occupation) return 1;
	if((x < 0) || (y < 0) || (x >= m_width) || (y >= m_height)) return 1;
	return m_occupation[x][y];
}

void Level::setOccupation(int x, int y, int occupied)
{
	if(!m_occupation) return;
	if((x < 0) || (y < 0) || (x >= m_width) || (y >= m_height)) return;

	if(occupied) m_occupation[x][y]++;
	else m_occupation[x][y]--;
}

