#include "handler.h"
#include "level.h"
#include "players.h"
#include "protocol.h"
#include <stdlib.h>
#include <iostream>
#include "config.h"

using namespace std;

static Handler *o;

Handler::Handler()
{
	level = NULL;
	players = new Players();
	o = this;
}

Handler::~Handler()
{
	if(level) delete level;
	delete players;
}

void Handler::event(GGZdMod *ggzdmod, GGZdModEvent event, void *data) // static !!! pass object?
{
	int player;
	int spectator;

	cout << "================"<< endl;
	cout << "Got event: " << event << endl;

	if((event == GGZDMOD_EVENT_JOIN)
	|| (event == GGZDMOD_EVENT_LEAVE)
	|| (event == GGZDMOD_EVENT_PLAYER_DATA))
	{
		player = *(int*)data;
		cout << "* Associated player: " << player << " (" <<
			ggzdmod_get_seat(ggzdmod, player).name << ")" << endl;
	}

	if((event == GGZDMOD_EVENT_SPECTATOR_JOIN)
	|| (event == GGZDMOD_EVENT_SPECTATOR_LEAVE))
	{
		spectator = *(int*)data;
		cout << "* Associated spectator: " << spectator << " (" <<
			ggzdmod_get_spectator(ggzdmod, spectator).name << ")" << endl;
	}

	switch(event)
	{
		case GGZDMOD_EVENT_STATE:
			cout << "* Type: state change" << endl;
			break;
		case GGZDMOD_EVENT_JOIN:
			cout << "* Type: join" << endl;
			// TODO: send complete map to player, update map, send updates to
			// all players including the new one
			o->players->add(player);
			break;
		case GGZDMOD_EVENT_LEAVE:
			// TODO: remove player from map, send updates to all remaining
			cout << "* Type: leave" << endl;
			o->players->remove(player);
			break;
		case GGZDMOD_EVENT_PLAYER_DATA:
			//: TODO: moves etc. here or in TICK
			cout << "* Type: player data" << endl;
			break;
		case GGZDMOD_EVENT_SPECTATOR_JOIN:
			break;
		case GGZDMOD_EVENT_SPECTATOR_LEAVE:
			break;
	}
}

void Handler::loop()
{
	GGZdMod *ggzdmod;
	int ret;

	level = new Level();
	level->loadLevel(DATADIR "/levels/level.txt");

	ggzdmod = ggzdmod_new(GGZDMOD_GAME);

	ggzdmod_set_handler(ggzdmod, GGZDMOD_EVENT_STATE, &event);
	ggzdmod_set_handler(ggzdmod, GGZDMOD_EVENT_JOIN, &event);
	ggzdmod_set_handler(ggzdmod, GGZDMOD_EVENT_LEAVE, &event);
	ggzdmod_set_handler(ggzdmod, GGZDMOD_EVENT_PLAYER_DATA, &event);
	ggzdmod_set_handler(ggzdmod, GGZDMOD_EVENT_SPECTATOR_JOIN, &event);
	ggzdmod_set_handler(ggzdmod, GGZDMOD_EVENT_SPECTATOR_LEAVE, &event);

	ggzdmod_connect(ggzdmod);

	ret = ggzdmod_loop(ggzdmod);

	if(ret)
	{
		cerr << "Error: Couldn't connect to ggzd." << endl;
	}
}

